@@ -3,5 +3,5 @@
 """Top-level package for navarp."""
 
 __author__ = """Federico Bisti"""
-__email__ = 'fbisti@cells.es'
-__version__ = '0.18.0'
+__email__ = 'federico.bisti@univaq.it'
+__version__ = '1.0.0'
