@@ -34,9 +34,9 @@ from scipy import linalg
 
 from matplotlib.patches import Polygon
 
-from navarp.utils import ktransf, isocut
+from navarp.utils import ktransf
 
-#from joblib import Parallel, delayed
+# from joblib import Parallel, delayed
 
 
 def get_isoen_from_kxy_interp(kxy_interp, kx, ky, isoen, mask=np.array([])):
@@ -52,7 +52,7 @@ def get_isoen_from_kxy_interp(kxy_interp, kx, ky, isoen, mask=np.array([])):
             faster is the interpolation procedure).
 
     Return:
-        isoen_interp (ndarray): The interpolated values at kxy_interp points.
+        isoen_interp (ndarray): The interpolated data at kxy_interp points.
     """
     # Construct isoen_interp
     isoen_interp = np.full(kxy_interp.shape[0], np.nan)
@@ -115,7 +115,7 @@ def get_isoen_from_kx_ky_interp(kx_interp, ky_interp, kx, ky, isoen):
         isoen (ndarray): The isoen map related to kx and ky.
 
     Return:
-        isoen_interp (ndarray): The interpolated values at the grid formed
+        isoen_interp (ndarray): The interpolated data at the grid formed
             from kx_interp and ky_interp.
     """
     # Construct kxy_interp
@@ -132,12 +132,12 @@ def get_isoen_from_kx_ky_interp(kx_interp, ky_interp, kx, ky, isoen):
     return isoen_interp
 
 
-def get_isoen(bins_x, bins_y, kx, ky, isoen):
+def get_isoen(kxbins, kybins, kx, ky, isoen):
     """Interpolate isoen on the generated uniform binned 2D-grid
 
     Args:
-        bins_x (int): defines the number of equal-width bins in the given kx
-        bins_y (int): defines the number of equal-width bins in the given ky
+        kxbins (int): defines the number of equal-width bins in the given kx
+        kybins (int): defines the number of equal-width bins in the given ky
         kx (ndarray): The kx vector where the isoen is defined;
         ky (ndarray): The ky vector where the isoen is defined;
         isoen (ndarray): The isoen map related to kx and ky.
@@ -148,11 +148,11 @@ def get_isoen(bins_x, bins_y, kx, ky, isoen):
               sampled;
             * **ky_interp** (ndarray): The ky vector where the isoen is
               sampled;
-            * **isoen_interp** (ndarray): The interpolated values at the
+            * **isoen_interp** (ndarray): The interpolated data at the
               uniform grid.
     """
-    kx_interp = np.linspace(kx.min(), kx.max(), bins_x)
-    ky_interp = np.linspace(ky.min(), ky.max(), bins_y)
+    kx_interp = np.linspace(kx.min(), kx.max(), kxbins)
+    ky_interp = np.linspace(ky.min(), ky.max(), kybins)
 
     isoen_interp = get_isoen_from_kx_ky_interp(
         kx_interp,
@@ -165,12 +165,12 @@ def get_isoen(bins_x, bins_y, kx, ky, isoen):
     return kx_interp, ky_interp, isoen_interp
 
 
-def get_isok_kxy_interp(k_pts_xy, bins):
+def get_isok_kxy_interp(k_pts_xy, kbins):
     """Generate uniform binned 1D path passing througt the k_pts
 
     Args:
         k_pts_xy (ndarray): list of (kx, ky) points defining the path;
-        bins (int): defines the number of equal-width bins in the given path.
+        kbins (int): defines the number of equal-width bins in the given path.
 
     Returns:
         (tuple):
@@ -189,19 +189,19 @@ def get_isok_kxy_interp(k_pts_xy, bins):
     # buikd ky_interp
     num_pts = len(k_pts_xy)
     num_segment = num_pts - 1
-    bins += num_segment - 1
-    bins_rest = bins
+    kbins += num_segment - 1
+    kbins_rest = kbins
     k_pts_bin = np.zeros(num_pts, dtype=int)
     k_pts_bin[0] = 0
     for ind in range(num_segment):
-        # define the bins for this segment
-        bins_norm = int(distances[ind]/tot_dist * bins)
+        # define the kbins for this segment
+        kbins_norm = int(distances[ind]/tot_dist * kbins)
         if ind != num_segment-1:
-            bins_rest -= bins_norm
-            bin_pt = bins - bins_rest - (ind + 1)
+            kbins_rest -= kbins_norm
+            bin_pt = kbins - kbins_rest - (ind + 1)
         else:
-            bins_norm = bins_rest
-            bin_pt = bins - (ind + 1)
+            kbins_norm = kbins_rest
+            bin_pt = kbins - (ind + 1)
         # save location of the point
         k_pts_bin[ind+1] = bin_pt
 
@@ -209,7 +209,7 @@ def get_isok_kxy_interp(k_pts_xy, bins):
         x_p0, y_p0 = k_pts_xy[ind]
         x_p1, y_p1 = k_pts_xy[ind+1]
 
-        rho = np.linspace(0., distances[ind], bins_norm)
+        rho = np.linspace(0., distances[ind], kbins_norm)
         azim = np.arctan2(y_p1 - y_p0, x_p1 - x_p0)
 
         kxy_interp = rho[:, None] * [np.cos(azim), np.sin(azim)] + [x_p0, y_p0]
@@ -309,34 +309,14 @@ def get_isok_mask(k_pts_xy, kx, ky):
     return mask
 
 
-def get_unique_isok_mask(
-    ens_interp,
-    k_pts_xy,
-    entry,
-    tht_an,
-    phi_an,
-    k_perp_slit_for_kz,
-    inn_pot,
-    p_hv,
-    efermis=0
-):
-    """Generate a unique mask (boolean matrix) for all the ekins_interp range
+def get_unique_isok_mask(ebins_interp, k_pts_xy, entry):
+    """Generate a unique mask (boolean matrix) for all the ebins_interp range
 
     Args:
-        ens_interp (ndarray): The energies vector to sample the
+        ebins_interp (ndarray): The binding energies vector to sample the
             entry.data at;
         k_pts_xy (ndarray): list of (kx, ky) points defining the path;
-        entry (NavEntry class): the class for the data explored by NavARP;
-        tht_an (float): angle between analyzer axis (a) and normal (n) to the
-            surface along the plane of the slit (theta, so the name "tht_an");
-        phi_an (float): angle between analyzer axis (a) and normal (n) along
-            the plane perpendicular to the slit (phi, so the name "phi_an"),
-            this is the axis of the scan in the case of tilt rotation;
-        k_perp_slit_p (float): k vector value perpendicular to the slit, in the
-            case of scan_type=='hv' (usually = 0);
-        inn_pot (float): Inner potential, corresponding to the energy of the
-            bottom of the valence band referenced to vacuum level;
-        p_hv (boolean): If True, add photon momentum.
+        entry (NavEntry class): the class for the data explored by NavARP.
 
     Returns:
         mask_final (ndarray of booleans): It is the region where the
@@ -344,8 +324,8 @@ def get_unique_isok_mask(
             faster is the interpolation procedure), for every energy.
     """
 
-    ei_min = ens_interp.min()
-    ei_max = ens_interp.max()
+    ei_min = ebins_interp.min()
+    ei_max = ebins_interp.max()
     ei_delta = ei_max - ei_min
     ens_mask = [
         ei_min,
@@ -361,14 +341,8 @@ def get_unique_isok_mask(
 
     mask_final = np.array([])
     for en_mask in ens_mask:
-        kx, ky = ktransf.get_k_isoen(
-            entry,
-            en_mask+efermis,
-            tht_an,
-            phi_an,
-            k_perp_slit_for_kz,
-            inn_pot,
-            p_hv)
+        kx, ky = ktransf.get_k_isoen(entry, en_mask+entry.efermi)
+
         mask = get_isok_mask(k_pts_xy, kx, ky)
 
         if mask_final.size > 0:
@@ -384,189 +358,88 @@ def get_unique_isok_mask(
 
 
 def get_isok(
-    ekins_interp,
-    k_pts_xy,
-    bins,
     entry,
-    tht_an,
-    phi_an,
-    k_perp_slit_for_kz=0,
-    inn_pot=14,
-    p_hv=False,
+    kx_pts,
+    ky_pts,
+    ebins_interp=None,
+    kbins=None,
     mask_once=True
-):
-    """Interpolate entry.data along the k_pts_xy points path for ekins_interp
-
-    Args:
-        ekins_interp (ndarray): The kinetic energies vector to sample the
-            entry.data at;
-        k_pts_xy (ndarray): list of (kx, ky) points defining the path;
-        bins (int): defines the number of equal-width bins in the given path;
-        entry (NavEntry class): the class for the data explored by NavARP;
-        tht_an (float): angle between analyzer axis (a) and normal (n) to the
-            surface along the plane of the slit (theta, so the name "tht_an");
-        phi_an (float): angle between analyzer axis (a) and normal (n) along
-            the plane perpendicular to the slit (phi, so the name "phi_an"),
-            this is the axis of the scan in the case of tilt rotation;
-        k_perp_slit_p (float): k vector value perpendicular to the slit, in the
-            case of scan_type=='hv' (usually = 0);
-        inn_pot (float): Inner potential, corresponding to the energy of the
-            bottom of the valence band referenced to vacuum level;
-        p_hv (boolean, optional): If True, add photon momentum.
-        mask_once (boolean, optional): If True, the region (mask), where the
-            interpolation function is going to be defined (the smaller the
-            faster is the interpolation procedure), is generated once and used
-            in all the ekins_interp range.
-
-    Returns:
-        (tuple):
-            * **krho** (ndarray): the distance vector from the first point;
-            * **kxy_interp** (ndarray): the (kx, ky) points along the path;
-            * **isok_interp** (ndarray): The interpolated values along the
-              path;
-            * **k_pts_bin** (ndarray of int): the index values defining the
-              segments along the path.
-    """
-
-    krho, kxy_interp, k_pts_bin = get_isok_kxy_interp(k_pts_xy, bins)
-
-    if mask_once:
-        mask = get_unique_isok_mask(
-            ekins_interp,
-            k_pts_xy,
-            entry,
-            tht_an,
-            phi_an,
-            k_perp_slit_for_kz,
-            inn_pot,
-            p_hv
-        )
-
-    isok_interp = np.full((krho.shape[0], ekins_interp.shape[0]), np.nan)
-    dekin = abs(ekins_interp[1] - ekins_interp[0])*0.5
-    for i, ekin in enumerate(ekins_interp):
-        isoen = isocut.maps_sum(ekin, dekin, entry.energies, entry.data)
-
-        kx, ky = ktransf.get_k_isoen(
-            entry,
-            ekin,
-            tht_an,
-            phi_an,
-            k_perp_slit_for_kz,
-            inn_pot,
-            p_hv
-        )
-
-        if not mask_once:
-            mask = get_isok_mask(k_pts_xy, kx, ky)
-
-        isok_interp[:, i] = get_isoen_from_kxy_interp(
-            kxy_interp,
-            kx,
-            ky,
-            isoen,
-            mask=mask
-        )
-
-    return krho, kxy_interp, isok_interp, k_pts_bin
-
-
-def get_isok_from_ebins(
-    ebins_interp,
-    k_pts_xy,
-    bins,
-    e_bins,
-    data_algn,
-    efermis,
-    entry,
-    tht_an,
-    phi_an,
-    k_perp_slit_for_kz=0,
-    inn_pot=14,
-    p_hv=False,
-    mask_once=False
 ):
     """Interpolate entry.data along the k_pts_xy points path for ebins_interp
 
     Args:
+        entry (NavEntry class): the class for the data explored by NavARP;
+        kx_pts (ndarray): kx coordinate of the points defining the path;
+        ky_pts (ndarray): ky coordinate of the points defining the path;
         ebins_interp (ndarray): The binding energies vector to sample the
             entry.data at;
-        k_pts_xy (ndarray): list of (kx, ky) points defining the path;
-        bins (int): defines the number of equal-width bins in the given path;
-        entry (NavEntry class): the class for the data explored by NavARP;
-        tht_an (float): angle between analyzer axis (a) and normal (n) to the
-            surface along the plane of the slit (theta, so the name "tht_an");
-        phi_an (float): angle between analyzer axis (a) and normal (n) along
-            the plane perpendicular to the slit (phi, so the name "phi_an"),
-            this is the axis of the scan in the case of tilt rotation;
-        k_perp_slit_p (float): k vector value perpendicular to the slit, in the
-            case of scan_type=='hv' (usually = 0);
-        inn_pot (float): Inner potential, corresponding to the energy of the
-            bottom of the valence band referenced to vacuum level;
-        p_hv (boolean, optional): If True, add photon momentum.
+        kbins (int): defines the number of equal-width bins in the given path;
         mask_once (boolean, optional): If True, the region (mask), where the
             interpolation function is going to be defined (the smaller the
             faster is the interpolation procedure), is generated once and used
-            in all the ekins_interp range.
+            in all the ebins_interp range.
 
     Returns:
         (tuple):
             * **krho** (ndarray): the distance vector from the first point;
             * **kxy_interp** (ndarray): the (kx, ky) points along the path;
-            * **isok_interp** (ndarray): The interpolated values along the path.
+            * **ebins_interp** (ndarray): The interpolated data along the path;
+            * **isok_interp** (ndarray): The interpolated data along the path;
             * **k_pts_bin** (ndarray of int): the index values defining the
               segments along the path.
     """
 
-    krho, kxy_interp, k_pts_bin = get_isok_kxy_interp(k_pts_xy, bins)
+    k_pts_xy = np.append(
+        ktransf.asarray(kx_pts)[:, None],
+        ktransf.asarray(ky_pts)[:, None],
+        axis=1
+    )
+
+    if ebins_interp is None:
+        ebins_interp = entry.ebins
 
     if mask_once:
-        mask = get_unique_isok_mask(
-            ebins_interp,
-            k_pts_xy,
-            entry,
-            tht_an,
-            phi_an,
-            k_perp_slit_for_kz,
-            inn_pot,
-            p_hv,
-            efermis=efermis
+        mask = get_unique_isok_mask(ebins_interp, k_pts_xy, entry)
+
+    if kbins is None:
+        # get delta_kx
+        isoen = entry.isoenergy(ebins_interp.mean())
+        delta_kx = (isoen.kx.max() - isoen.kx.min()) / len(isoen.angles)
+
+        # get total distance
+        distances = np.sqrt(
+            np.diff(k_pts_xy[:, 0])**2 + np.diff(k_pts_xy[:, 1])**2
         )
+        tot_dist = np.sum(distances)
+
+        kbins = int((tot_dist / delta_kx)*1.2)
+
+    krho, kxy_interp, k_pts_bin = get_isok_kxy_interp(k_pts_xy, kbins)
 
     isok_interp = np.full((krho.shape[0], ebins_interp.shape[0]), np.nan)
     debin = abs(ebins_interp[1] - ebins_interp[0])*0.5
     for i, ebin in enumerate(ebins_interp):
-        isoen = isocut.maps_sum(ebin, debin, e_bins, data_algn)
-
-        kx, ky = ktransf.get_k_isoen(
-            entry,
-            ebin+efermis,
-            tht_an,
-            phi_an,
-            k_perp_slit_for_kz,
-            inn_pot,
-            p_hv
-        )
+        isoen = entry.isoenergy(ebin, debin)
 
         if not mask_once:
-            mask = get_isok_mask(k_pts_xy, kx, ky)
+            mask = get_isok_mask(k_pts_xy, isoen.kx, isoen.ks)
 
         isok_interp[:, i] = get_isoen_from_kxy_interp(
             kxy_interp,
-            kx,
-            ky,
-            isoen,
+            isoen.kx,
+            isoen.ks,
+            isoen.data,
             mask=mask
         )
 
-    return krho, kxy_interp, isok_interp, k_pts_bin
+    return krho, kxy_interp, ebins_interp, isok_interp, k_pts_bin
 
 
 def get_isoscan(
     kx,
     energies,
     isoscan,
-    bins,
+    kbins,
     kind="cubic",
     fill_value="extrapolate",
     assume_sorted=True
@@ -577,7 +450,7 @@ def get_isoscan(
         kx (ndarray): The k vector where the isoscan is defined
         energies (ndarray): The energy vector where the isoscan is defined
         isoscan (ndarray): The isoscan map related to kx and energies
-        bins (int): defines the number of equal-width bins in the given vector
+        kbins (int): defines the number of equal-width bins in the given vector
         kind(str or int, optional): Specifies the kind of interpolation
             as a string ('linear', 'nearest', 'zero', 'slinear', 'quadratic',
             'cubic', 'previous', 'next', where 'zero', 'slinear', 'quadratic'
@@ -603,10 +476,10 @@ def get_isoscan(
         (tuple):
             * **kx_interp** (ndarray): The kx vector where the isoen is
               sampled;
-            * **isoen_interp** (ndarray): The interpolated values at the
+            * **isoen_interp** (ndarray): The interpolated data at the
               uniform binned kx_interp vector.
     """
-    kx_interp = np.linspace(kx.min(), kx.max(), bins)
+    kx_interp = np.linspace(kx.min(), kx.max(), kbins)
 
     isoscan_interp = get_isoscan_from_kx_interp(
         kx_interp,
@@ -659,7 +532,7 @@ def get_isoscan_from_kx_interp(
             order and they are sorted first. If True, x has to be an array of
             monotonically increasing values.
     Return:
-        isoen_interp (ndarray): The interpolated values at kx_interp and
+        isoen_interp (ndarray): The interpolated data at kx_interp and
             energies.
     """
     isoscan_interp = np.zeros((kx_interp.shape[0], energies.shape[0]))
@@ -677,42 +550,42 @@ def get_isoscan_from_kx_interp(
     return isoscan_interp
 
 
-#def get_data_from_isoscan_interp(
-#    kx_interp,
-#    kx,
-#    energies,
-#    data,
-#    kind="nearest"
-#):
-#    """Generate data as collection of interpolated isoscan in uniform grid
-#
-#    Args:
-#        kx_interp (ndarray): The uniform k vector to sample the data at
-#        kx (ndarray): The k vector where the data are defined
-#        energies (ndarray): The energy vector where the data are defined
-#        data (ndarray): The data related to kx and energies
-#        kind(str or int, optional): Specifies the kind of interpolation
-#            as a string ('linear', 'nearest', 'zero', 'slinear', 'quadratic',
-#            'cubic', 'previous', 'next', where 'zero', 'slinear', 'quadratic'
-#            and 'cubic' refer to a spline interpolation of zeroth, first,
-#            second or third order; 'previous' and 'next' simply return the
-#            previous or next value of the point) or as an integer specifying
-#            the order of the spline interpolator to use.
-#
-#    Return:
-#        (ndarray): The interpolated values at kx_interp and energies.
-#    """
-#    pool = Parallel(n_jobs=-1, pre_dispatch='all')
-#
-#    return pool(
-#        delayed(
-#            get_isoscan_from_kx_interp)(
-#                kx_interp=kx_interp,
-#                kx=kx,
-#                energies=energies,
-#                isoscan=data[scan_i, :, :],
-#                kind=kind,
-#                fill_value="extrapolate",
-#                assume_sorted=True
-#            ) for scan_i in range(data.shape[0])
-#    )
+# def get_data_from_isoscan_interp(
+#     kx_interp,
+#     kx,
+#     energies,
+#     data,
+#     kind="nearest"
+# ):
+#     """Generate data as collection of interpolated isoscan in uniform grid
+
+#     Args:
+#         kx_interp (ndarray): The uniform k vector to sample the data at
+#         kx (ndarray): The k vector where the data are defined
+#         energies (ndarray): The energy vector where the data are defined
+#         data (ndarray): The data related to kx and energies
+#         kind(str or int, optional): Specifies the kind of interpolation
+#             as a string ('linear', 'nearest', 'zero', 'slinear', 'quadratic',
+#             'cubic', 'previous', 'next', where 'zero', 'slinear', 'quadratic'
+#             and 'cubic' refer to a spline interpolation of zeroth, first,
+#             second or third order; 'previous' and 'next' simply return the
+#             previous or next value of the point) or as an integer specifying
+#             the order of the spline interpolator to use.
+
+#     Return:
+#         (ndarray): The interpolated data at kx_interp and energies.
+#     """
+#     pool = Parallel(n_jobs=-1, pre_dispatch='all')
+
+#     return pool(
+#         delayed(
+#             get_isoscan_from_kx_interp)(
+#                 kx_interp=kx_interp,
+#                 kx=kx,
+#                 energies=energies,
+#                 isoscan=data[scan_i, :, :],
+#                 kind=kind,
+#                 fill_value="extrapolate",
+#                 assume_sorted=True
+#             ) for scan_i in range(data.shape[0])
+#     )
